﻿using UnityEngine;
using System.Collections;

//using System;
using UnityEditor;
using System.Security.Policy;

public class MapGenerator : MonoBehaviour
{
	public int Width;
	public int Height;
	public string Seed;
	public bool UseRandomSeed;

	// ランダムに埋めるパーセンテージ
	[Range (0, 100)]
	public int RandomFillPercent;
	int[,] map;

	void Start ()
	{
		GenerateMap ();
	}

	void Update ()
	{
		if (Input.GetMouseButtonDown (0)) {
			GenerateMap ();
		}
	}

	void GenerateMap ()
	{
		map = new int[Width, Height];
		Debug.Log ("GenerateMap");

		RandomFillMap ();

		// スムーズにする。
		for (int i = 0; i < 5; i++) {
			SmoothMap ();
		}

		MeshGenerator meshGenerator = GetComponent<MeshGenerator> ();
		meshGenerator.GenerateMesh (map, 1);
	}

	void RandomFillMap ()
	{
		if (UseRandomSeed) {
			Seed = Time.time.ToString ();
			Debug.Log ("Seed: " + Seed);
		}
		// C# System.Random.
		System.Random psuedoRandom = new System.Random (Seed.GetHashCode ());
		// UnityEngine.Random.
		//Random.seed = Seed.GetHashCode ();

		// generate map data.
		for (int x = 0; x < Width; x++) {
			for (int y = 0; y < Height; y++) {
				if (x == 0 || x == Width - 1 || y == 0 || y == Height - 1) {
					map [x, y] = 1; // wall;
				} else {
					// #C System.Random.
					map [x, y] = (psuedoRandom.Next (0, 100) < RandomFillPercent) ? 1 : 0;
					// UnityEngine.Random.
					//map [x, y] = (Random.Range (0, 100) < RandomFillPercent) ? 1 : 0;
				}
			}
		}
	}

	void SmoothMap ()
	{
		for (int x = 0; x < Width; x++) {
			for (int y = 0; y < Height; y++) {
				int neighbourWallTiles = GetSurroundingWallCount (x, y);
				if (neighbourWallTiles > 4) // TODO: 4 is Magic Number!!
					map [x, y] = 1;
				else if (neighbourWallTiles < 4)
					map [x, y] = 0;
			}
		}
	}

	/**
	 * 3x3のマス目の中心をgridX,gridYとして周りの壁を数える。
	 */
	int GetSurroundingWallCount (int gridX, int gridY)
	{
		int wallCount = 0;
		for (int neighbourX = gridX - 1; neighbourX <= gridX + 1; neighbourX++) {
			for (int neighbourY = gridY - 1; neighbourY <= gridY + 1; neighbourY++) {
				if (neighbourX >= 0 && neighbourX < Width && neighbourY >= 0 && neighbourY < Height) {
					if (neighbourX != gridX || neighbourY != gridY) {
						wallCount += map [neighbourX, neighbourY];
					}
				} else {
					wallCount++;
				}
			}
		}
		return wallCount;
	}

	void OnDrawGizmos ()
	{
		/*
		for (int x = 0; x < Width; x++) {
			for (int y = 0; y < Height; y++) {
				if (map == null) {
					continue;
				}
				Gizmos.color = (map [x, y] == 1) ? Color.black : Color.white;
				Vector3 pos = new Vector3 (-Width / 2 + x + .5f, 0, -Height / 2 + y + .5f);
				Gizmos.DrawCube (pos, Vector3.one);
			}
		}
		*/
	}
}
